// Copyright (c) 2008  Future Technology Devices International Ltd.
//
// Module Name
//
//     FT2232HI2CUnit.pas
//
// Abstract:
//
//    Delphi program, to test the FTCI2C.DLL. The FTCI2C.DLL is used to control the FT2232H dual hi-speed device
//    and the FT4232H quad hi-speed device to simulate the Inter-Integrated Circuit(I2C) synchronous protocol to
//    communicate with I2C devices connected to FT2232H dual hi-speed devices and FT4232H quad hi-speed devices.
//
// Revision History:
//
//   09/09/08    Kirke Ashton    Created.
//
unit FT2232HI2CUnit;

interface

uses
  Windows;

type FTC_STATUS = Integer;

const MAX_NUM_DEVICE_NAME_CHARS = 100;

type TDeviceName = array [0..(MAX_NUM_DEVICE_NAME_CHARS - 1)] of Char;
type PDeviceName = ^TDeviceName;

const MAX_NUM_CHANNEL_CHARS = 5;

type TChannel = array [0..(MAX_NUM_CHANNEL_CHARS - 1)] of Char;
type PChannel = ^TChannel;

type HiSpeedDeviceTypes = (FT2232H_DEVICE_TYPE = 1, FT4232H_DEVICE_TYPE = 2);

type FtcInputOutputPins = record
  Pin1InputOutputState: LongBool;
  Pin1LowHighState: LongBool;
  Pin2InputOutputState: LongBool;
  Pin2LowHighState: LongBool;
  Pin3InputOutputState: LongBool;
  Pin3LowHighState: LongBool;
  Pin4InputOutputState: LongBool;
  Pin4LowHighState: LongBool;
end;

type PFtcInputOutputPins = ^FtcInputOutputPins;

type FthInputOutputPins = record
  Pin1InputOutputState: LongBool;
  Pin1LowHighState: LongBool;
  Pin2InputOutputState: LongBool;
  Pin2LowHighState: LongBool;
  Pin3InputOutputState: LongBool;
  Pin3LowHighState: LongBool;
  Pin4InputOutputState: LongBool;
  Pin4LowHighState: LongBool;
  Pin5InputOutputState: LongBool;
  Pin5LowHighState: LongBool;
  Pin6InputOutputState: LongBool;
  Pin6LowHighState: LongBool;
  Pin7InputOutputState: LongBool;
  Pin7LowHighState: LongBool;
  Pin8InputOutputState: LongBool;
  Pin8LowHighState: LongBool;
end;

type PFthInputOutputPins = ^FthInputOutputPins;

type FtcLowHighPins = record
  Pin1LowHighState: LongBool;
  Pin2LowHighState: LongBool;
  Pin3LowHighState: LongBool;
  Pin4LowHighState: LongBool;
end;

type PFtcLowHighPins = ^FtcLowHighPins;

type FthLowHighPins = record
  Pin1LowHighState: LongBool;
  Pin2LowHighState: LongBool;
  Pin3LowHighState: LongBool;
  Pin4LowHighState: LongBool;
  Pin5LowHighState: LongBool;
  Pin6LowHighState: LongBool;
  Pin7LowHighState: LongBool;
  Pin8LowHighState: LongBool;
end;

type PFthLowHighPins = ^FthLowHighPins;

const MAX_WRITE_CONTROL_BYTES_BUFFER_SIZE = 255;    // 256 bytes

type WriteControlByteBuffer = array[0..(MAX_WRITE_CONTROL_BYTES_BUFFER_SIZE - 1)] of Byte;
type PWriteControlByteBuffer = ^WriteControlByteBuffer;

type FtcPageWriteData = record
  dwNumPages: Dword;
  dwNumBytesPerPage: Dword;
end;

type PFtcPageWriteData = ^FtcPageWriteData;

const MAX_WRITE_DATA_BYTES_BUFFER_SIZE = 65535;    // 64k bytes

type WriteDataByteBuffer = array[0..(MAX_WRITE_DATA_BYTES_BUFFER_SIZE - 1)] of Byte;
type PWriteDataByteBuffer = ^WriteDataByteBuffer;

const MAX_READ_DATA_BYTES_BUFFER_SIZE = 65535;    // 64k bytes

type ReadDataByteBuffer = array[0..(MAX_READ_DATA_BYTES_BUFFER_SIZE - 1)] of Byte;
type PReadDataByteBuffer = ^ReadDataByteBuffer;

const MAX_NUM_DLL_VERSION_CHARS = 10;

type TDllVersion = array [0..(MAX_NUM_DLL_VERSION_CHARS - 1)] of Char;
type PDllVersion = ^TDllVersion;

const MAX_NUM_ERROR_MESSAGE_CHARS = 100;

type TErrorMessage = array [0..(MAX_NUM_ERROR_MESSAGE_CHARS - 1)] of Char;
type PErrorMessage = ^TErrorMessage;

type TFT2232HI2C = class
 public
  constructor create;
  function  GetNumHiSpeedDevices(pNumHiSpeedDevices: PDword): FTC_STATUS;
  function  GetHiSpeedDeviceNameLocIDChannel(DeviceNameIndex: Dword; pDeviceNameBuffer: PDeviceName; DeviceNameBufferSize: Dword; pLocationID: PDword; pChannelBuffer: PChannel; ChannelBufferSize: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS;
  function  OpenHiSpeedDevice(DeviceName: String; LocationID: Dword; Channel: String; pFtHandle: PDword): FTC_STATUS;
  function  GetHiSpeedDeviceType(fthandle: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS; 
  function  Close(fthandle: Dword): FTC_STATUS;
  function  InitDevice(fthandle: Dword; ClockDivisor: Dword): FTC_STATUS;
  function  GetHiSpeedDeviceClock(ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS;
  function  SetClock(fthandle: Dword; ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS;
  function  SetLoopback(fthandle: Dword; bLoopbackState: LongBool): FTC_STATUS;
  function  SetMode(fthandle: Dword; CommsMode: Dword): FTC_STATUS;
  function  SetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowInputOutputPins: PFtcInputOutputPins; bControlHighInputOutputPins: LongBool; HighInputOutputPins: PFthInputOutputPins): FTC_STATUS;
  function  GetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowPinsInputData: PFtcLowHighPins; bControlHighInputOutputPins: LongBool; HighPinsInputData: PFthLowHighPins): FTC_STATUS;
  function  Write(fthandle: Dword; pWriteControlBuffer: PWriteControlByteBuffer; NumControlBytesToWrite: Dword; bControlAcknowledge: LongBool; ControlAckTimeoutSecs: Dword; bStopCondition: LongBool; DataWriteTypes: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumDataBytesToWrite: Dword; bDataAcknowledge: LongBool; DataAckTimeoutSecs: Dword; pPageWriteData: PFtcPageWriteData): FTC_STATUS;
  function  Read(fthandle: Dword; pWriteControlBuffer: PWriteControlByteBuffer; NumControlBytesToWrite: Dword; bControlAcknowledge: LongBool; ControlAckTimeoutSecs: Dword; DataReadTypes: Dword; pReadDataBuffer: PReadDataByteBuffer; NumDataBytesToRead: Dword): FTC_STATUS;
  function  GetDllVersion(DllVersionBuffer: pDllVersion; BufferSize: Dword): FTC_STATUS;
  function  GetErrorCodeString(Language: String; StatusCode: FTC_STATUS; pErrorMessageBuffer: PErrorMessage; BufferSize: Dword): FTC_STATUS;
end;


const
  FT2232HI2C_DLL_Name = 'ftci2c.dll';

  NO_WRITE_TYPE = 0;
  BYTE_WRITE_TYPE = 1;
  PAGE_WRITE_TYPE = 2;

  BYTE_READ_TYPE = 1;
  BLOCK_READ_TYPE = 2;

  STANDARD_MODE = 1;
  FAST_MODE = 2;

  FTC_SUCCESS = 0;

  ENGLISH = 'EN';

  
implementation

function I2C_GetNumHiSpeedDevices(pNumHiSpeedDevices: PDword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_GetNumHiSpeedDevices';
function I2C_GetHiSpeedDeviceNameLocIDChannel(DeviceNameIndex: Dword; pDeviceNameBuffer: PDeviceName; DeviceNameBufferSize: Dword; pLocationID: PDword; pChannelBuffer: PChannel; ChannelBufferSize: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_GetHiSpeedDeviceNameLocIDChannel';
function I2C_OpenHiSpeedDevice(DeviceName: String; LocationID: Dword; Channel: String; pFtHandle: PDword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_OpenHiSpeedDevice';
function I2C_GetHiSpeedDeviceType(fthandle: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_GetHiSpeedDeviceType';
function I2C_Close(fthandle: Dword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_Close';
function I2C_InitDevice(fthandle: Dword; ClockFrequencyValue: Dword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_InitDevice';
function I2C_GetHiSpeedDeviceClock(ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_GetHiSpeedDeviceClock';
function I2C_SetClock(fthandle: Dword; ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_SetClock';
function I2C_SetLoopback(fthandle: Dword; bLoopbackState: LongBool): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_SetLoopback';
function I2C_SetMode(fthandle: Dword; CommsMode: Dword): FTC_STATUS;  stdcall ; External FT2232HI2C_DLL_Name name 'I2C_SetMode';
function I2C_SetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowInputOutputPins: PFtcInputOutputPins; bControlHighInputOutputPins: LongBool; HighInputOutputPins: PFthInputOutputPins): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_SetHiSpeedDeviceGPIOs';
function I2C_GetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool; LowPinsInputData: PFtcLowHighPins; bControlHighInputOutputPins: LongBool; HighPinsInputData: PFthLowHighPins): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_GetHiSpeedDeviceGPIOs';
function I2C_Write(fthandle: Dword; pWriteControlBuffer: PWriteControlByteBuffer; NumControlBytesToWrite: Dword; bControlAcknowledge: LongBool; ControlAckTimeoutSecs: Dword; bStopCondition: LongBool; DataWriteTypes: Dword; pWriteDataBuffer: PWriteDataByteBuffer; NumDataBytesToWrite: Dword; bDataAcknowledge: LongBool; DataAckTimeoutSecs: Dword; pPageWriteData: PFtcPageWriteData): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_Write';
function I2C_Read(fthandle: Dword; pWriteControlBuffer: PWriteControlByteBuffer; NumControlBytesToWrite: Dword; bControlAcknowledge: LongBool; ControlAckTimeoutSecs: Dword; DataReadTypes: Dword; pReadDataBuffer: PReadDataByteBuffer; NumDataBytesToRead: Dword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_Read';
function I2C_GetDllVersion(DllVersionBuffer: pDllVersion; BufferSize: Dword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_GetDllVersion';
function I2C_GetErrorCodeString(Language: PChar; StatusCode: FTC_STATUS; ErrorMessageBuffer: PErrorMessage; BufferSize: Dword): FTC_STATUS; stdcall ; External FT2232HI2C_DLL_Name name 'I2C_GetErrorCodeString';


constructor TFT2232HI2C.create;
begin
end;

function  TFT2232HI2C.GetNumHiSpeedDevices(pNumHiSpeedDevices: PDword): FTC_STATUS;
begin
  Result := I2C_GetNumHiSpeedDevices(pNumHiSpeedDevices);
end;

function  TFT2232HI2C.GetHiSpeedDeviceNameLocIDChannel(DeviceNameIndex: Dword; pDeviceNameBuffer: PDeviceName;
                                                       DeviceNameBufferSize: Dword; pLocationID: PDword;
                                                       pChannelBuffer: PChannel; ChannelBufferSize: Dword;
                                                       pHiSpeedDeviceType: PDword): FTC_STATUS;
begin
  Result := I2C_GetHiSpeedDeviceNameLocIDChannel(DeviceNameIndex, pDeviceNameBuffer, DeviceNameBufferSize, pLocationID, pChannelBuffer, ChannelBufferSize, pHiSpeedDeviceType);
end;

function  TFT2232HI2C.OpenHiSpeedDevice(DeviceName: String; LocationID: Dword; Channel: String; pFtHandle: PDword): FTC_STATUS;
begin
  Result := I2C_OpenHiSpeedDevice(DeviceName, LocationID, Channel, pFtHandle);
end;

function  TFT2232HI2C.GetHiSpeedDeviceType(fthandle: Dword; pHiSpeedDeviceType: PDword): FTC_STATUS;
begin
  Result := I2C_GetHiSpeedDeviceType(fthandle, pHiSpeedDeviceType);
end;

function  TFT2232HI2C.Close(fthandle: Dword): FTC_STATUS;
begin
  Result := I2C_Close(fthandle);
end;

function  TFT2232HI2C.InitDevice(fthandle: Dword; ClockDivisor: Dword): FTC_STATUS;
begin
  Result := I2C_InitDevice(fthandle, ClockDivisor);
end;

function  TFT2232HI2C.GetHiSpeedDeviceClock(ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS;
begin
  Result := I2C_GetHiSpeedDeviceClock(ClockDivisor, pClockFrequencyHz);
end;

function  TFT2232HI2C.SetClock(fthandle: Dword; ClockDivisor: Dword; pClockFrequencyHz: PDword): FTC_STATUS;
begin
  Result :=  I2C_SetClock(fthandle, ClockDivisor, pClockFrequencyHz);
end;

function  TFT2232HI2C.SetLoopback(fthandle: Dword; bLoopbackState: LongBool): FTC_STATUS;
begin
  Result := I2C_SetLoopback(fthandle, bLoopbackState);
end;

function  TFT2232HI2C.SetMode(fthandle: Dword; CommsMode: Dword): FTC_STATUS;
begin
  Result := I2C_SetMode(fthandle, CommsMode);
end;

function  TFT2232HI2C.SetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool;
                                            LowInputOutputPins: PFtcInputOutputPins;
                                            bControlHighInputOutputPins: LongBool;
                                            HighInputOutputPins: PFthInputOutputPins): FTC_STATUS;
begin
  Result := I2C_SetHiSpeedDeviceGPIOs(fthandle, bControlLowInputOutputPins, LowInputOutputPins, bControlHighInputOutputPins, HighInputOutputPins);
end;

function  TFT2232HI2C.GetHiSpeedDeviceGPIOs(fthandle: Dword; bControlLowInputOutputPins: LongBool;
                                            LowPinsInputData: PFtcLowHighPins;
                                            bControlHighInputOutputPins: LongBool;
                                            HighPinsInputData: PFthLowHighPins): FTC_STATUS;
begin
  Result := I2C_GetHiSpeedDeviceGPIOs(fthandle, bControlLowInputOutputPins, LowPinsInputData, bControlHighInputOutputPins, HighPinsInputData);
end;

function  TFT2232HI2C.Write(fthandle: Dword; pWriteControlBuffer: PWriteControlByteBuffer; NumControlBytesToWrite: Dword;
                            bControlAcknowledge: LongBool; ControlAckTimeoutSecs: Dword;
                            bStopCondition: LongBool; DataWriteTypes: Dword;
                            pWriteDataBuffer: PWriteDataByteBuffer; NumDataBytesToWrite: Dword;
                            bDataAcknowledge: LongBool; DataAckTimeoutSecs: Dword;
                            pPageWriteData: PFtcPageWriteData): FTC_STATUS;
begin
  Result := I2C_Write(fthandle, pWriteControlBuffer, NumControlBytesToWrite, bControlAcknowledge, ControlAckTimeoutSecs, bStopCondition, DataWriteTypes, pWriteDataBuffer, NumDataBytesToWrite, bDataAcknowledge, DataAckTimeoutSecs, pPageWriteData);
end;

function  TFT2232HI2C.Read(fthandle: Dword; pWriteControlBuffer: PWriteControlByteBuffer;
                           NumControlBytesToWrite: Dword; bControlAcknowledge: LongBool;
                           ControlAckTimeoutSecs: Dword; DataReadTypes: Dword;
                           pReadDataBuffer: PReadDataByteBuffer; NumDataBytesToRead: Dword): FTC_STATUS;
begin
  Result := I2C_Read(fthandle, pWriteControlBuffer, NumControlBytesToWrite, bControlAcknowledge, ControlAckTimeoutSecs, DataReadTypes, pReadDataBuffer, NumDataBytesToRead);
end;

function  TFT2232HI2C.GetDllVersion(DllVersionBuffer: pDllVersion; BufferSize: Dword): FTC_STATUS;
begin
  Result := I2C_GetDllVersion(DllVersionBuffer, BufferSize);
end;

function  TFT2232HI2C.GetErrorCodeString(Language: String; StatusCode: FTC_STATUS; pErrorMessageBuffer: PErrorMessage; BufferSize: Dword): FTC_STATUS;
begin
  Result := I2C_GetErrorCodeString(PChar(Language), StatusCode, pErrorMessageBuffer, BufferSize);
end;

end.
