// Copyright (c) 2008  Future Technology Devices International Ltd.
//
// Module Name
//
//     I2CMain.pas
//
// Abstract:
//
//    Delphi program, to test the FTCI2C.DLL. The FTCI2C.DLL is used to control the FT2232H dual hi-speed device
//    and the FT4232H quad hi-speed device to simulate the Inter-Integrated Circuit(I2C) synchronous protocol to
//    communicate with I2C devices connected to FT2232H dual hi-speed devices and FT4232H quad hi-speed devices.
//
// Revision History:
//
//   09/09/08    Kirke Ashton    Created.
//
unit I2CMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, D2xxunit, DSI2C, ComCtrls, DateUtils, I2CDLLFuncUnit,
  FT2232HI2CUnit;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    DeviceNameComboBox: TComboBox;
    Panel2: TPanel;
    Memo1: TMemo;
    Panel3: TPanel;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Edit1: TEdit;
    Label1: TLabel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    Label2: TLabel;
    UpDown1: TUpDown;
    Edit2: TEdit;
    Label3: TLabel;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Button4: TButton;
    ComboBox2: TComboBox;
    Label4: TLabel;
    ByteValueEdit: TEdit;
    FreqLabel: TLabel;
    ByteValueLabel: TLabel;
    PageWriteCheckBox: TCheckBox;
    BlockReadCheckBox: TCheckBox;
    ExternalDeviceLabel: TLabel;
    ExternalDeviceComboBox: TComboBox;
    FastStandardModeCheckBox: TCheckBox;
    Exit1: TMenuItem;
    HiSpeedDeviceTypeRadioGroup: TRadioGroup;
    FT2232HRadioButton: TRadioButton;
    FT4232HRadioButton: TRadioButton;
    GPIOLPanel: TPanel;
    GPIOHPanel: TPanel;
    GPIOLPinsLabel: TLabel;
    GPIOLBitLabel: TLabel;
    GPIOLBit2Label: TLabel;
    GPIOLBit2OutputCheckBox: TCheckBox;
    GPIOLBit2HighCheckBox: TCheckBox;
    GPIOLBit3OutputCheckBox: TCheckBox;
    GPIOLBit3HighCheckBox: TCheckBox;
    GPIOLBit4Label: TLabel;
    GPIOLBit4OutputCheckBox: TCheckBox;
    GPIOLBit4HighCheckBox: TCheckBox;
    GPIOLValueReadEdit: TEdit;
    GPIOLValueReadLabel: TLabel;
    GPIOHPinsLabel: TLabel;
    GPIOHBitLabel: TLabel;
    GPIOHBit1Label: TLabel;
    GPIOHBit1OutputCheckBox: TCheckBox;
    GPIOHBit1HighCheckBox: TCheckBox;
    GPIOHBit2Label: TLabel;
    GPIOHBit2OutputCheckBox: TCheckBox;
    GPIOHBit2HighCheckBox: TCheckBox;
    GPIOHBit3Label: TLabel;
    GPIOHBit3OutputCheckBox: TCheckBox;
    GPIOHBit3HighCheckBox: TCheckBox;
    GPIOHBit4Label: TLabel;
    GPIOHBit4OutputCheckBox: TCheckBox;
    GPIOHBit4HighCheckBox: TCheckBox;
    GPIOHBit5Label: TLabel;
    GPIOHBit5OutputCheckBox: TCheckBox;
    GPIOHBit5HighCheckBox: TCheckBox;
    GPIOHBit6Label: TLabel;
    GPIOHBit6OutputCheckBox: TCheckBox;
    GPIOHBit6HighCheckBox: TCheckBox;
    GPIOHBit7Label: TLabel;
    GPIOHBit7OutputCheckBox: TCheckBox;
    GPIOHBit7HighCheckBox: TCheckBox;
    GPIOHBit8Label: TLabel;
    GPIOHBit8OutputCheckBox: TCheckBox;
    GPIOHBit8HighCheckBox: TCheckBox;
    GPIOHValueReadEdit: TEdit;
    GPIOLUpdateLowButton: TButton;
    GPIOHValueReadLabel: TLabel;
    GPIOHUpdateLowButton: TButton;
     
    procedure DeviceNameComboBoxDropDown(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure Button3Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure N1Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure UpDown1Changing(Sender: TObject; var AllowChange: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure Edit2Exit(Sender: TObject);
    procedure Edit2Change(Sender: TObject);
    procedure ExternalDeviceComboBoxChange(Sender: TObject);
    procedure FastStandardModeCheckBoxClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure DeviceNameComboBoxChange(Sender: TObject);
    procedure GPIOLUpdateLowButtonClick(Sender: TObject);
    procedure GPIOHUpdateLowButtonClick(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

Type
  Names = Array[1..20] of string;
  Names_Ptr = ^Names;

var
  Form1: TForm1;
  My_Names : Names;

  BinFileName : TfileName;
  BinFile: file;
  ImageData : ARRAY[0..127] of word;
  DevAddress :integer;

implementation

{$R *.DFM}

procedure GetNames;
var
  i : integer;
  DeviceNames: TStringList;
begin
  Form1.DeviceNameComboBox.Clear;
  if (FT2232HI2CDll = True) then
  begin
    DeviceNames := TStringList.Create;

    if GetI2CDeviceNames(DeviceNames) then
    begin
      for i := 0 to (DeviceNames.Count - 1) do
        Form1.DeviceNameComboBox.Items.Add(DeviceNames[i]);
    end;

    DeviceNames.Free;
  end
  else
  begin
    for i := 1 to 20 do My_Names[i] := '';
    List_Devs(@My_Names);
    i := 1;
    if (My_Names[i] <> '' ) then
    begin
      repeat
      Form1.DeviceNameComboBox.Items.Add(My_Names[i]);
      i := i + 1;
      until (My_Names[i] = '');
    end;
  end;
end;

procedure TForm1.DeviceNameComboBoxChange(Sender: TObject);
begin
  if (HiSpeedDevices[form1.DeviceNameComboBox.ItemIndex].HiSpeedDeviceType = Dword(FT2232H_DEVICE_TYPE)) then
    form1.FT2232HRadioButton.Checked := true
  else
    form1.FT4232HRadioButton.Checked := true;
end;

procedure TForm1.DeviceNameComboBoxDropDown(Sender: TObject);
begin
GetNames;
end;


function Check_For_Dev_A_And_B(Name : string) : boolean;
var len,i : integer;
SubString : string;
begin
Check_For_Dev_A_And_B := false;
if (Name <> '') then
  begin
  len := length(Name);
  if (len > 2) then
    begin
    SubString := Name[len-1] + Name[len];
    if ((SubString = ' A') or (SubString = ' B')) then
      Check_For_Dev_A_And_B := true;
    end;
  end;
end;

function Program_Device(bErase: Boolean): Boolean;
var
  DeviceName, ExternalDevice : string;
  DisplayLines: TStringList;
  i: Integer;
begin
  DeviceName := Trim(form1.DeviceNameComboBox.Text);
  ExternalDevice := Trim(form1.ExternalDeviceComboBox.Text);

  if ((DeviceName <> '') and (ExternalDevice <> '')) then
  begin
    DevAddress := strtoint(Form1.ComboBox2.Text);

    form1.Memo1.Clear;
    form1.Memo1.Update;
    
    DisplayLines := TStringList.Create;

    Program_External_Device(DeviceName, Form1.DeviceNameComboBox.ItemIndex, form1.ExternalDeviceComboBox.ItemIndex,
                            strtoint(Trim(form1.Edit2.Text)), DevAddress, bErase,
                            Form1.PageWriteCheckBox.Checked, Trim(Form1.ByteValueEdit.Text),
                            DisplayLines);

    for i := 0 to (DisplayLines.Count - 1) do
      form1.Memo1.Lines.Add(DisplayLines[i]);

    DisplayLines.Free;
  end
  else
  begin
    if (DeviceName = '') then
      form1.Memo1.Lines.Add('No valid device selected in drop down window.')
    else
      form1.Memo1.Lines.Add('No valid external device selected in drop down window.')
  end;

  Result := True; 
end;

procedure Program_To_Address;
var passed : boolean;
i,j,data : integer;
DeviceName : string;
HexByteValueStr, ByteValueStr: string;
begin
DevAddress := strtoint(Form1.ComboBox2.Text);
DeviceName := form1.DeviceNameComboBox.Text;
  form1.Memo1.Clear;
  form1.Memo1.Update;

if (FT2232HI2CDll = True) then
  Program_Device(False)
else
begin
passed := Check_For_Dev_A_And_B(DeviceName);
if passed then
  begin
  passed := Init_Controller(DeviceName);
  if passed then
    begin
    ByteValueStr := Trim(Form1.ByteValueEdit.Text);
    for j := 0 to $7F do
      begin
      if (ByteValueStr = '') then
        data := j
      else
      begin
        HexByteValueStr := '$';
        HexByteValueStr := HexByteValueStr + ByteValueStr;

        data := StrToInt(HexByteValueStr);
      end;
      passed := Write_Location(j,DevAddress,data);
      form1.Memo1.Lines.Add('Addr : ' + HexWrdToStr(j)+' Data : ' + HexWrdToStr(data) );
      end;
    ClosePort;
    end;
  end
else
  begin
  form1.Memo1.Lines.Add('You must select the "A" or "B" port of the chip to use the MPSSE');
  end;
end;
end;

procedure Prog_From_File;
var passed : boolean;
i,j : integer;
DName : string;
begin
DevAddress := strtoint(Form1.ComboBox2.Text);
DName := form1.DeviceNameComboBox.Text;
passed := Check_For_Dev_A_And_B(DName);
if passed then
  begin
  passed := Init_Controller(DName);
  if passed then
    begin
    for j := 0 to $7F do
      begin
      passed := Write_Location(j,DevAddress,imageData[j]);
      form1.Memo1.Lines.Add('Addr : ' + HexWrdToStr(j)+' Data : ' + HexWrdToStr(j) );
      end;
    ClosePort;
    end;
  end
else
  begin
  form1.Memo1.Lines.Add('You must select the "A" or "B" port of the chip to use the MPSSE');
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var passed : boolean;
i,j : integer;
DName, ExternalDevice, tst : string;
BlockDataRead: ReadBlockDataBuff;
DisplayLines: TStringList;
begin
  dsi2c.speed := strtoint(form1.Edit2.Text);
  DName := form1.DeviceNameComboBox.Text;
  passed := Check_For_Dev_A_And_B(DName);
  if passed then
  begin
    DevAddress := strtoint(Form1.ComboBox2.Text);
    
    form1.Memo1.Clear;
    form1.Memo1.Update;

    if (FT2232HI2CDll = True) then
    begin
      ExternalDevice := Trim(form1.ExternalDeviceComboBox.Text);
      if ((DName <> '') and (ExternalDevice <> '')) then
      begin
        DisplayLines := TStringList.Create;

        Read_External_Device(DName, Form1.DeviceNameComboBox.ItemIndex, form1.ExternalDeviceComboBox.ItemIndex,
                             strtoint(Trim(form1.Edit2.Text)), DevAddress, Form1.BlockReadCheckBox.Checked,
                             DisplayLines);

         for i := 0 to (DisplayLines.Count - 1) do
           form1.Memo1.Lines.Add(DisplayLines[i]);

        DisplayLines.Free;
      end
      else
      begin
        if (DName = '') then
          form1.Memo1.Lines.Add('No valid device selected in drop down window.')
        else
          form1.Memo1.Lines.Add('No valid external device selected in drop down window.')
      end;
    end
    else
    begin
      passed := Init_Controller(DName);
      if passed then
      begin
        if (not Form1.BlockReadCheckBox.Checked) then
        begin
          for j := 0 to $7F do
          begin
            i := Read_Location(j,DevAddress);
            i := i AND $FFFF;
            imageData[j] := i;
            form1.Memo1.Lines.Add('Addr : ' + HexWrdToStr(j)+' Data : ' + HexWrdToStr(i) );
          end;
        end
        else
        begin
          ReadBlockData(0, DevAddress, @BlockDataRead, 128);

          for j := 0 to $7F do
            form1.Memo1.Lines.Add('Addr : ' + HexWrdToStr(j)+' Data : ' + HexWrdToStr(BlockDataRead[j]));
        end;
        ClosePort;
      end;
    end;
  end
  else
  begin
    form1.Memo1.Lines.Add('You must select the "A" or "B" port of the chip to use the MPSSE');
  end;
end;

procedure update_clk_divisor;
begin
//form1.Edit2.Text := inttostr(form1.UpDown1.Position);
dsi2c.speed := strtoint(form1.Edit2.Text);
end;

procedure TForm1.UpDown1Click(Sender: TObject; Button: TUDBtnType);
begin
//update_clk_divisor;
end;

procedure TForm1.Button3Click(Sender: TObject);
var passed : boolean;
DName : string;
begin
DevAddress := strtoint(Form1.ComboBox2.Text);
dsi2c.speed := strtoint(form1.Edit2.Text);
DName := form1.DeviceNameComboBox.Text;
passed := Check_For_Dev_A_And_B(DName);
if passed then
  begin
  if (FT2232HI2CDll = True) then
    passed := Program_Device(True)
  else
    passed := Erase_24C02(DName,DevAddress);
  if passed then
    form1.Memo1.Lines.Add('Erase - Success')
  else
    form1.Memo1.Lines.Add('Erase - Failed');
  end
else
  begin
  form1.Memo1.Lines.Add('You must select the "A" or "B" port of the chip to use the MPSSE');
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
Program_To_Address;
end;

procedure Read_In_Array;
var cnt : integer;
ByteData : ARRAY[0..255] of byte;
begin
Form1.OpenDialog1.FileName := '';
Form1.OpenDialog1.execute;
if Form1.OpenDialog1.FileName <> '' then
  begin
    form1.Edit1.Text := Form1.OpenDialog1.FileName;
    try
    BinFileName := Form1.OpenDialog1.FileName;
    AssignFile(BinFile,BinFileName);
    reset(BinFile,1);
    blockread(BinFile,ByteData,256,Cnt);
    CloseFile(BinFile);
    finally
    end;
  for cnt := 0 to 127 do
    ImageData[cnt] := (ByteData[(Cnt * 2) + 1] * 256 ) OR (ByteData[Cnt * 2]);
  end;
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
Read_In_Array;
end;

procedure TForm1.N1Click(Sender: TObject);
var Cnt : integer;
ByteData : ARRAY[0..255] of byte;
begin
for cnt := 0 to 127 do
  begin
  ByteData[(Cnt * 2)] := ImageData[cnt] AND $FF;
  ByteData[(Cnt * 2) + 1] := ImageData[cnt] DIV 256;
  end;
Form1.SaveDialog1.FileName := '';
Form1.SaveDialog1.Execute;
if Form1.SaveDialog1.FileName <> '' then
  begin
    form1.Edit1.Text := Form1.SaveDialog1.FileName;
    try
    BinFileName := Form1.SaveDialog1.FileName;
    AssignFile(BinFile,BinFileName);
    rewrite(BinFile,1);
    blockwrite(BinFile,ByteData,256,Cnt);
    CloseFile(BinFile);
    finally
    end;
  end;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
Read_In_Array;
Prog_From_File;
end;

procedure TForm1.UpDown1Changing(Sender: TObject;
  var AllowChange: Boolean);
begin
//update_clk_divisor;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  I2CChipIndex: Dword;
begin
  form1.UpDown1.Position := MaxFreq24LC16BClockDivisor;
  form1.Edit2.Text := inttostr(form1.UpDown1.Position);
  update_clk_divisor;

  Form1.ExternalDeviceComboBox.Clear;
  for I2CChipIndex := 0 to (NumI2CChips - 1) do
    Form1.ExternalDeviceComboBox.Items.Add(I2CChips[I2CChipIndex]);

  FT2232HI2CDll := True;
  FT2232HI2C := TFT2232HI2C.Create;
  ftHandle := 0;
  CommsMode := FAST_MODE;
end;

procedure TForm1.GPIOHUpdateLowButtonClick(Sender: TObject);
var
  passed : boolean;
  DName : string;
  ftStatus: FTC_STATUS;
  LowInputOutputPins: FtcInputOutputPins;
  LowPinsInputData: FtcLowHighPins;
  HighInputOutputPins: FthInputOutputPins;
  HighPinsInputData: FthLowHighPins;
  HighPinsReadValue: Integer;
begin
  DName := form1.DeviceNameComboBox.Text;
  passed := Check_For_Dev_A_And_B(DName);
  if passed then
  begin
    GPIOHValueReadEdit.Text := '';

    HighInputOutputPins.Pin1InputOutputState := GPIOHBit1OutputCheckBox.Checked;
    HighInputOutputPins.Pin1LowHighState := GPIOHBit1HighCheckBox.Checked;
    HighInputOutputPins.Pin2InputOutputState := GPIOHBit2OutputCheckBox.Checked;
    HighInputOutputPins.Pin2LowHighState := GPIOHBit2HighCheckBox.Checked;
    HighInputOutputPins.Pin3InputOutputState := GPIOHBit3OutputCheckBox.Checked;
    HighInputOutputPins.Pin3LowHighState := GPIOHBit3HighCheckBox.Checked;
    HighInputOutputPins.Pin4InputOutputState := GPIOHBit4OutputCheckBox.Checked;
    HighInputOutputPins.Pin4LowHighState := GPIOHBit4HighCheckBox.Checked;
    HighInputOutputPins.Pin5InputOutputState := GPIOHBit5OutputCheckBox.Checked;
    HighInputOutputPins.Pin5LowHighState := GPIOHBit5HighCheckBox.Checked;
    HighInputOutputPins.Pin6InputOutputState := GPIOHBit6OutputCheckBox.Checked;
    HighInputOutputPins.Pin6LowHighState := GPIOHBit6HighCheckBox.Checked;
    HighInputOutputPins.Pin7InputOutputState := GPIOHBit7OutputCheckBox.Checked;
    HighInputOutputPins.Pin7LowHighState := GPIOHBit7HighCheckBox.Checked;
    HighInputOutputPins.Pin8InputOutputState := GPIOHBit8OutputCheckBox.Checked;
    HighInputOutputPins.Pin8LowHighState := GPIOHBit8HighCheckBox.Checked;

    ftStatus := SetGetHiSpeedGPIOs(Form1.DeviceNameComboBox.ItemIndex, strtoint(Trim(form1.Edit2.Text)),
                                   False, @LowInputOutputPins, @LowPinsInputData,
                                   True, @HighInputOutputPins, @HighPinsInputData);

    if (ftStatus = FTC_SUCCESS) then
    begin
      HighPinsReadValue := 0;

      if (HighPinsInputData.Pin1LowHighState = True) then
      begin
        HighPinsReadValue := 1;
      end;
      if (HighPinsInputData.Pin2LowHighState = True) then
      begin
        HighPinsReadValue := HighPinsReadValue + 2;
      end;
      if (HighPinsInputData.Pin3LowHighState = True) then
      begin
        HighPinsReadValue := HighPinsReadValue + 4;
      end;
      if (HighPinsInputData.Pin4LowHighState = True) then
      begin
        HighPinsReadValue := HighPinsReadValue + 8;
      end;
      if (HighPinsInputData.Pin5LowHighState = True) then
      begin
        HighPinsReadValue := HighPinsReadValue + 16;
      end;
      if (HighPinsInputData.Pin6LowHighState = True) then
      begin
        HighPinsReadValue := HighPinsReadValue + 32;
      end;
      if (HighPinsInputData.Pin7LowHighState = True) then
      begin
        HighPinsReadValue := HighPinsReadValue + 64;
      end;
      if (HighPinsInputData.Pin8LowHighState = True) then
      begin
        HighPinsReadValue := HighPinsReadValue + 128;
      end;

      GPIOHValueReadEdit.Text := intToStr(HighPinsReadValue);
    end;
  end
  else
  begin
    form1.Memo1.Lines.Add('You must select the "A" or "B" port of the chip to use the MPSSE');
  end;
end;

procedure TForm1.GPIOLUpdateLowButtonClick(Sender: TObject);
var
  passed : boolean;
  DName : string;
  ftStatus: FTC_STATUS;
  LowInputOutputPins: FtcInputOutputPins;
  LowPinsInputData: FtcLowHighPins;
  HighInputOutputPins: FthInputOutputPins;
  HighPinsInputData: FthLowHighPins;
  LowPinsReadValue: Integer;
begin
  DName := form1.DeviceNameComboBox.Text;
  passed := Check_For_Dev_A_And_B(DName);
  if passed then
  begin
    GPIOLValueReadEdit.Text := '';
  
    LowInputOutputPins.Pin1InputOutputState := False;
    LowInputOutputPins.Pin1LowHighState := False;
    LowInputOutputPins.Pin2InputOutputState := GPIOLBit2OutputCheckBox.Checked;
    LowInputOutputPins.Pin2LowHighState := GPIOLBit2HighCheckBox.Checked;
    LowInputOutputPins.Pin3InputOutputState := GPIOLBit3OutputCheckBox.Checked;
    LowInputOutputPins.Pin3LowHighState := GPIOLBit3HighCheckBox.Checked;
    LowInputOutputPins.Pin4InputOutputState := GPIOLBit4OutputCheckBox.Checked;
    LowInputOutputPins.Pin4LowHighState := GPIOLBit4HighCheckBox.Checked;

    ftStatus := SetGetHiSpeedGPIOs(Form1.DeviceNameComboBox.ItemIndex, strtoint(Trim(form1.Edit2.Text)),
                                   True, @LowInputOutputPins, @LowPinsInputData,
                                  False, @HighInputOutputPins, @HighPinsInputData);

    if (ftStatus = FTC_SUCCESS) then
    begin
      LowPinsReadValue := 0;

      if (LowPinsInputData.Pin2LowHighState = True) then
      begin
        LowPinsReadValue := 2;
      end;
      if (LowPinsInputData.Pin3LowHighState = True) then
      begin
        LowPinsReadValue := LowPinsReadValue + 4;
      end;
      if (LowPinsInputData.Pin4LowHighState = True) then
      begin
        LowPinsReadValue := LowPinsReadValue + 8;
      end;

      GPIOLValueReadEdit.Text := intToStr(LowPinsReadValue);
    end;
  end
  else
  begin
    form1.Memo1.Lines.Add('You must select the "A" or "B" port of the chip to use the MPSSE');
  end;
end;

procedure TForm1.Edit2Exit(Sender: TObject);
begin
if (form1.Edit2.Text <> '') then
  begin
  form1.UpDown1.Position := strtoint(form1.edit2.Text );
  update_clk_divisor;
  end;
end;

procedure TForm1.Edit2Change(Sender: TObject);
var
  ftStatus: FTC_STATUS;
  ClockDivisor, ClockFrequencyHz: Dword;
begin
if (form1.Edit2.Text <> '') then
  begin
  //form1.UpDown1.Position := strtoint(form1.edit2.Text );
  update_clk_divisor;

  ClockDivisor := strtoint(Trim(form1.Edit2.Text));
  ftStatus := FT2232HI2C.GetHiSpeedDeviceClock(ClockDivisor, @ClockFrequencyHz);

  if (ftStatus = FTC_SUCCESS) then
    Form1.FreqLabel.Caption := FrequencyLabelDefaultText + '(' + IntToStr(ClockFrequencyHz) + 'Hz)'
  else
    DisplayFT2232HI2CDllError('Edit2Change', 'GetClock', ftStatus);

  end;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  form1.Close();

  // Release destroy's the form and free its associated memory. Release does
  // not destroy the form until all event handlers of the form and event
  // handlers of components on the form have finished executing. Any event
  // handlers of the form should use Release instead of Free. Failing to do
  // so could lead to an access violation
  form1.Release();
end;

procedure TForm1.ExternalDeviceComboBoxChange(Sender: TObject);
begin
  Form1.ComboBox2.ItemIndex := form1.ExternalDeviceComboBox.ItemIndex;

  case form1.ExternalDeviceComboBox.ItemIndex of
    I2C24LC16BChipIndex: form1.UpDown1.Position := MaxFreq24LC16BClockDivisor;
    I2CM24C64ChipIndex: form1.UpDown1.Position := MaxFreqM24C64ClockDivisor;
    I2CDS1621ChipIndex: form1.UpDown1.Position := MaxFreqDS1621ClockDivisor;
    I2CM41T81ChipIndex: form1.UpDown1.Position := MaxFreqM41T81ClockDivisor;
  end;

  form1.Edit2.Text := inttostr(form1.UpDown1.Position);
end;

procedure TForm1.FastStandardModeCheckBoxClick(Sender: TObject);
begin
  if (FT2232HI2CDll = True) then
  begin
    CommsMode := FAST_MODE;
    if FastStandardModeCheckBox.Checked then
      CommsMode := STANDARD_MODE;
  end;
end;

end.
